package com.yihu.ehr.resource.client;

import com.yihu.ehr.agModel.resource.RsReportCategoryInfoModel;
import com.yihu.ehr.agModel.resource.RsReportCategoryModel;
import com.yihu.ehr.constants.ApiVersion;
import com.yihu.ehr.constants.MicroServices;
import com.yihu.ehr.constants.ServiceApi;
import com.yihu.ehr.model.resource.MRsReportCategory;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

/**
 * 资源报表分类 client
 *
 * @author 张进军
 * @created 2017.8.8 19:22
 */
@FeignClient(name = MicroServices.Resource)
@RequestMapping(value = ApiVersion.Version1_0)
@ApiIgnore
public interface RsReportCategoryClient {

    @ApiOperation("根据ID获取资源报表分类")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategory, method = RequestMethod.GET)
    MRsReportCategory getById(
            @ApiParam(name = "id", value = "主键", required = true)
            @PathVariable(value = "id") Integer id);

    @ApiOperation(value = "根据条件获取资源报表分类")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategories, method = RequestMethod.GET)
    List<MRsReportCategory> search(
            @ApiParam(name = "codeName", value = "资源报表分类编码或名称")
            @RequestParam(value = "codeName", required = false) String codeName);

    @ApiOperation(value = "获取资源报表分类的树形下拉框数据")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryComboTree, method = RequestMethod.GET)
    public List<MRsReportCategory> getComboTreeData();

    @ApiOperation("新增资源报表分类")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategorySave, method = RequestMethod.POST)
    MRsReportCategory add(
            @ApiParam(name = "rsReportCategory", value = "资源报表分类JSON字符串", required = true)
            @RequestBody String rsReportCategory);

    @ApiOperation("更新资源报表分类")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategorySave, method = RequestMethod.PUT)
    MRsReportCategory update(
            @ApiParam(name = "rsReportCategory", value = "资源报表分类JSON字符串", required = true)
            @RequestBody String rsReportCategory);

    @ApiOperation("删除资源报表分类")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryDelete, method = RequestMethod.DELETE)
    void delete(
            @ApiParam(name = "id", value = "资源报表分类ID", required = true)
            @RequestParam(value = "id") Integer id);


    @ApiOperation("验证资源报表分类编码是否唯一")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryIsUniqueCode, method = RequestMethod.GET)
    Boolean isUniqueCode(
            @ApiParam(name = "id", value = "资源报表分类ID", required = true)
            @RequestParam("id") Integer id,
            @ApiParam(name = "code", value = "资源报表分类编码", required = true)
            @RequestParam("code") String code);

    @ApiOperation("验证资源报表分类名称是否唯一")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryIsUniqueName, method = RequestMethod.GET)
    Boolean isUniqueName(
            @ApiParam(name = "id", value = "资源报表分类ID", required = true)
            @RequestParam("id") Integer id,
            @ApiParam(name = "name", value = "资源报表分类名称", required = true)
            @RequestParam("name") String name);

    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryNoPageCategories, method = RequestMethod.GET)
    @ApiOperation("获取资源报表类别")
    List<MRsReportCategory> getAllCategories(
            @RequestParam(value="filters",required = false)String filters);

    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryByApp, method = RequestMethod.GET)
    @ApiOperation("获取平台应用对应的报表分类")
    List<RsReportCategoryModel> getCategoryByApp(@RequestParam(value = "appId") String appId);

    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryByIds, method = RequestMethod.GET)
    @ApiOperation("根据Id获取平台应用对应的报表分类")
    List<RsReportCategoryInfoModel> getCategoryByIds(
            @ApiParam(name = "ids", value = "id")
            @RequestParam(value = "ids") List<Integer> ids);

    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryIdsByCode, method = RequestMethod.GET)
    @ApiOperation("根据code获取平台应用对应的报表分类子类")
    List<Integer> getCategoryIdsByCode(
            @ApiParam(name = "code", value = "分类编码")
            @RequestParam(value = "code") String code);
}
