package com.yihu.ehr.users.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.ehr.agModel.app.AppFeatureModel;
import com.yihu.ehr.agModel.patient.PatientDetailModel;
import com.yihu.ehr.agModel.user.UserDetailModel;
import com.yihu.ehr.agModel.user.UsersModel;
import com.yihu.ehr.apps.service.AppClient;
import com.yihu.ehr.apps.service.AppFeatureClient;
import com.yihu.ehr.constants.AgAdminConstants;
import com.yihu.ehr.constants.ApiVersion;
import com.yihu.ehr.constants.ServiceApi;
import com.yihu.ehr.controller.BaseController;
import com.yihu.ehr.fileresource.service.FileResourceClient;
import com.yihu.ehr.geography.service.AddressClient;
import com.yihu.ehr.model.app.MApp;
import com.yihu.ehr.model.app.MAppFeature;
import com.yihu.ehr.model.app.MUserApp;
import com.yihu.ehr.model.dict.MConventionalDict;
import com.yihu.ehr.model.geography.MGeography;
import com.yihu.ehr.model.org.MOrgDeptData;
import com.yihu.ehr.model.org.MOrgDeptJson;
import com.yihu.ehr.model.org.MOrganization;
import com.yihu.ehr.model.patient.MDemographicInfo;
import com.yihu.ehr.model.security.MKey;
import com.yihu.ehr.model.user.MRoleFeatureRelation;
import com.yihu.ehr.model.user.MRoleUser;
import com.yihu.ehr.model.user.MUser;
import com.yihu.ehr.organization.service.OrgDeptMemberClient;
import com.yihu.ehr.organization.service.OrganizationClient;
import com.yihu.ehr.patient.controller.PatientController;
import com.yihu.ehr.patient.service.PatientClient;
import com.yihu.ehr.security.service.SecurityClient;
import com.yihu.ehr.systemdict.service.ConventionalDictEntryClient;
import com.yihu.ehr.users.service.RoleFeatureRelationClient;
import com.yihu.ehr.users.service.RoleUserClient;
import com.yihu.ehr.users.service.UserClient;
import com.yihu.ehr.util.datetime.DateTimeUtil;
import com.yihu.ehr.util.datetime.DateUtil;
import com.yihu.ehr.util.rest.Envelop;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.apache.xmlbeans.UserType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.netflix.feign.EnableFeignClients;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.regex.Pattern;

/**
 * Created by AndyCai on 2016/1/21.
 */
@EnableFeignClients
@RequestMapping(ApiVersion.Version1_0 + "/admin")
@RestController
@Api(value = "user", description = "用户管理", tags = {"用户管理"})
public class UserController extends BaseController {

    @Autowired
    private UserClient userClient;

    @Autowired
    private ConventionalDictEntryClient conventionalDictClient;

    @Autowired
    private OrganizationClient orgClient;

    @Autowired
    private SecurityClient securityClient;

    @Autowired
    private AddressClient addressClient;

    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private RoleFeatureRelationClient roleFeatureRelationClient;
    @Autowired
    AppFeatureClient appFeatureClient;
    @Autowired
    private RoleUserClient roleUserClient;
    @Autowired
    private FileResourceClient fileResourceClient;
    @Autowired
    private PatientClient patientClient;
    @Autowired
    private PatientController patientController;
    @Autowired
    private  AppClient appClient;
    @Autowired
    private OrgDeptMemberClient orgDeptMemberClient;


    private String resetFilter(String filters) {

        if (!StringUtils.isEmpty(filters)) {
            String orgName, orgCodes = "";
            boolean searchOrg = false;
            String[] values;
            String[] filterArr = filters.split(";");
            for (int i = 0; i < filterArr.length; i++) {
                String filter = filterArr[i];
                if (filter.startsWith("organization")) {
                    if (filter.contains("<>") || filter.contains(">=") || filter.contains("<=")) {
                        values = filter.substring(14).split(" ");
                    }else{
                        values = filter.substring(13).split(" ");
                    }
                    orgName = values[0];
                    ResponseEntity<List<MOrganization>> rs = orgClient.searchOrgs("", "fullName?" + orgName, "", 1000, 1);
                    if (rs.getStatusCode().value() <= 200) {
                        List<MOrganization> orgList = rs.getBody();
                        if (orgList.size() > 0) {
                            for (MOrganization org : orgList) {
                                orgCodes += "," + org.getOrgCode();
                            }
                            filterArr[i] = "organization=" + orgCodes.substring(1);
                        } else {
                            filterArr[i] = "organization=-1";
                        }
                        if (values.length > 1) {
                            filterArr[i] = filterArr[i] + " " + values[1];
                        }
                        searchOrg = true;
                        break;
                    } else {
                        throw new IllegalAccessError("解析错误");
                    }
                }
            }

            if (searchOrg) {
                filters = "";
                for (String filter : filterArr) {
                    filters += filter + ";";
                }
            }
        }
        return filters;
    }

    @RequestMapping(value = "/users", method = RequestMethod.GET)
    @ApiOperation(value = "获取用户列表", notes = "根据查询条件获取用户列表在前端表格展示")
    public Envelop searchUsers(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "id,name,secret,url,createTime")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "-createDate,-lastLoginTime,+name")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            @ApiParam(name = "orgCode", value = "机构编码", defaultValue = "")
            @RequestParam(value = "orgCode", required = false) String orgCode)throws Exception{
        if (null == sorts) {
            sorts = "-createDate,-lastLoginTime";
        }
        filters = resetFilter(filters);
        ResponseEntity<List<MUser>> responseEntity = userClient.searchUsers(fields, filters, sorts, size, page, orgCode);
        List<MUser> mUsers = responseEntity.getBody();
        List<UsersModel> usersModels = new ArrayList<>();
        for (MUser mUser : mUsers) {
            UsersModel usersModel = convertToModel(mUser, UsersModel.class);
            //usersModel.setLastLoginTime(DateToString( mUser.getLastLoginTime(),AgAdminConstants.DateTimeFormat));
            usersModel.setLastLoginTime(mUser.getLastLoginTime() == null ? "" : DateTimeUtil.simpleDateTimeFormat(mUser.getLastLoginTime()));

            if (StringUtils.isNotEmpty(mUser.getUserType())&& isInteger(mUser.getUserType())) {
                int userTypeInt = Integer.parseInt(mUser.getUserType().toString());
                Envelop envStr = roleUserClient.GetUserTypeById(userTypeInt);
                if(envStr != null){
                    HashMap userTypeM = objectMapper.readValue(toJson(envStr.getObj()),HashMap.class);
                    String userTypeName = userTypeM.get("name").toString();
                    usersModel.setUserTypeName(userTypeName);
                }
            }

            //获取用户来源信息
            if (StringUtils.isNotEmpty(mUser.getSource())) {
                MConventionalDict dict = conventionalDictClient.getUserSource(mUser.getSource());
                usersModel.setSourceName(dict == null ? "" : dict.getValue());
            }
            usersModels.add(usersModel);
        }

        //获取总条数
        int totalCount = getTotalCount(responseEntity);

//        String count = response.getHeader(AgAdminConstants.ResourceCount);
//        int totalCount = StringUtils.isNotEmpty(count) ? Integer.parseInt(count) : 0;

        Envelop envelop = getResult(usersModels, totalCount, page, size);
        return envelop;
    }

    @RequestMapping(value = "/users/admin/{user_id}", method = RequestMethod.DELETE)
    @ApiOperation(value = "删除用户", notes = "根据用户id删除用户")
    public Envelop deleteUser(
            @ApiParam(name = "user_id", value = "用户编号", defaultValue = "")
            @PathVariable(value = "user_id") String userId) {
        try {
            // 删除用户秘钥信息
            boolean _res = securityClient.deleteKeyByUserId(userId);

            if (!_res) {
                return failed("删除失败!");
            }

            boolean result = userClient.deleteUser(userId);
            if (!result) {
                return failed("删除失败!");
            }
            //删除用户-用户角色组关系
            //Todo 根据用户id删除的接口
            Collection<MRoleUser> mRoleUsers = roleUserClient.searchRoleUserNoPaging("userId=" + userId);
            if (mRoleUsers != null && mRoleUsers.size()>0) {
                StringBuffer buffer = new StringBuffer();
                for (MRoleUser m : mRoleUsers) {
                    buffer.append(m.getRoleId());
                    buffer.append(",");
                }
                String roleIds = buffer.substring(0, buffer.length() - 1);
                boolean bo = roleUserClient.batchDeleteRoleUserRelation(userId, roleIds);
                if (!bo) {
                    return failed("删除失败！");
                }
            }
            try {
                fileResourceClient.filesDelete(userId);
            } catch (Exception e) {
                return success("数据删除成功！头像图片删除失败！");
            }
            return success(null);
        } catch (Exception ex) {
            ex.printStackTrace();
            return failedSystem();
        }
    }


    @RequestMapping(value = "/user", method = RequestMethod.POST)
    @ApiOperation(value = "创建用户", notes = "重新绑定用户信息")
    public Envelop createUser(
            @ApiParam(name = "user_json_data", value = "", defaultValue = "")
            @RequestParam(value = "user_json_data",required = true) String userJsonData) {
        try {
            UserDetailModel detailModel = objectMapper.readValue(userJsonData, UserDetailModel.class);
            PatientDetailModel patientModel = objectMapper.readValue(userJsonData, PatientDetailModel.class);
            String idCard = detailModel.getIdCardNo();
            String userType = detailModel.getUserType();

            String errorMsg = null;
            if (StringUtils.isEmpty(detailModel.getLoginCode())) {
                errorMsg += "账户不能为空";
            }
            if (StringUtils.isEmpty(detailModel.getRealName())) {
                errorMsg += "姓名不能为空!";
            }
            if (StringUtils.isEmpty(idCard)) {
                errorMsg += "身份证号不能为空!";
            }
            if (StringUtils.isEmpty(detailModel.getTelephone())) {
                errorMsg += "电话号码不能为空!";
            }
            if (StringUtils.isNotEmpty(errorMsg)) {
                return failed(errorMsg);
            }
            if (userClient.isUserNameExists(detailModel.getLoginCode())) {
                return failed("账户已存在!");
            }
            if (userClient.isIdCardExists(idCard)) {
                return failed("身份证号已存在!");
            }
            if (userClient.isTelephoneExists(detailModel.getTelephone())) {
                return failed("电话号码已存在!");
            }

            //设置默认密码为身份证后八位
            if(!org.springframework.util.StringUtils.isEmpty(detailModel.getIdCardNo())&&detailModel.getIdCardNo().length()>9){
                String  defaultPassword=detailModel.getIdCardNo().substring(detailModel.getIdCardNo().length()-8,detailModel.getIdCardNo().length());
                detailModel.setPassword(defaultPassword);
            }else{
                detailModel.setPassword(AgAdminConstants.DefaultPassword);
            }
            //detailModel.setRole(null);
            MUser mUser = convertToMUser(detailModel);
//            增加居民注册账号时身份证号的校验，demographics表中已存在，users表增加demographic_id身份证号关联
            mUser.setDemographicId(idCard);
            if (!patientClient.isExistIdCardNo(idCard)) {
//                新增居民demographics表中居民信息
                String telephone = "{\"联系电话\":\"telephone\"}";
                telephone = telephone.replace("telephone", "" + detailModel.getTelephone());
                patientModel.setTelephoneNo(telephone);
                patientModel.setName(detailModel.getRealName());

                MDemographicInfo info = patientClient.createPatient(objectMapper.writeValueAsString(patientModel));
                if (info == null) {
                    return failed("保存失败!");
                }
            }

            mUser = userClient.createUser(objectMapper.writeValueAsString(mUser));
            if (mUser == null) {
                return failed("保存失败!");
            }
            //新增时先新增用户再保存所属角色组-人员关系表，用户新增失败（新增失败）、角色组关系表新增失败（删除新增用户-提示新增失败）
            if(!(detailModel.getRole() == null)&&StringUtils.isNotEmpty(detailModel.getRole())){
                String roles = detailModel.getRole();
                boolean bo = roleUserClient.batchCreateRolUsersRelation(mUser.getId(), roles);
                if (!bo) {
                    userClient.deleteUser(mUser.getId());
                    return failed("保存失败!");
                }
            }else{
                Envelop envelop = roleUserClient.setUserRolesForUpdate(mUser.getId(), Integer.parseInt(mUser.getUserType().toString()));
                if(!envelop.isSuccessFlg()){
                    return failed("用户新增成功，但授权失败，请重新手动进行授权配置！");
                }
            }
            detailModel = convertToUserDetailModel(mUser);
            return success(detailModel);
        } catch (Exception ex) {
            ex.printStackTrace();
            return failedSystem();
        }
    }


    @RequestMapping(value = "/user", method = RequestMethod.PUT)
    @ApiOperation(value = "用户管理界面 - 修改用户", notes = "重新绑定用户信息")
    public Envelop updateUser(
            @ApiParam(name = "user_json_data", value = "", defaultValue = "")
            @RequestParam(value = "user_json_data") String userJsonData) {
        try {
            UserDetailModel detailModel = toEntity(userJsonData, UserDetailModel.class);
            String errorMsg = "";
            if (StringUtils.isEmpty(detailModel.getLoginCode())) {
                errorMsg += "账户不能为空";
            }
            if (StringUtils.isEmpty(detailModel.getRealName())) {
                errorMsg += "姓名不能为空!";
            }
            if (StringUtils.isEmpty(detailModel.getIdCardNo())) {
                errorMsg += "身份证号不能为空!";
            }

            if (StringUtils.isEmpty(detailModel.getTelephone())) {
                errorMsg += "电话号码不能为空!";
            }

            if (StringUtils.isNotEmpty(errorMsg)) {
                return failed(errorMsg);
            }
            MUser mUser = userClient.getUser(detailModel.getId());
            if(mUser.getLoginCode() != null){
                if (!mUser.getLoginCode().equals(detailModel.getLoginCode())
                        && userClient.isUserNameExists(detailModel.getLoginCode())) {
                    return failed("账户已存在!");
                }
            }
            if(mUser.getIdCardNo() != null){
                if (mUser.getIdCardNo() != null && !mUser.getIdCardNo().equals(detailModel.getIdCardNo())
                        && userClient.isIdCardExists(detailModel.getIdCardNo())) {
                    return failed("身份证号已存在!");
                }
            }

            if(mUser.getTelephone() != null){
                if (!mUser.getTelephone().equals(detailModel.getTelephone())
                        && userClient.isTelephoneExists(detailModel.getTelephone())) {
                    return failed("电话号码已存在!");
                }
            }

            // 基于用户ID，删除所有的授权角色
            roleUserClient.deleteRoleUserBuUserId(mUser.getId());
            // 基于维护，进行授权的重新初始化,若未授权，则跳过
            if(!detailModel.getRole().isEmpty()){
                boolean bo = roleUserClient.batchCreateRolUsersRelation(mUser.getId(), detailModel.getRole());
                if (!bo) {
                    userClient.deleteUser(mUser.getId());
                    return failed("保存失败!");
                }
            }

            mUser = convertToMUser(detailModel);
            mUser.setRole(null);
            mUser = userClient.updateUser(objectMapper.writeValueAsString(mUser));
            if (mUser != null) {
                detailModel = convertToUserDetailModel(mUser);
                return success(detailModel);
            }else {
                return failed("保存失败！");
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            return failedSystem();
        }
    }


    @RequestMapping(value = "users/admin/{user_id}", method = RequestMethod.GET)
    @ApiOperation(value = "获取用户信息", notes = "包括地址信息等")
    public Envelop getUser(
            @ApiParam(name = "user_id", value = "", defaultValue = "")
            @PathVariable(value = "user_id") String userId) {

        try {
            MUser mUser = userClient.getUser(userId);
            if (mUser == null) {
                return failed("用户信息获取失败!");
            }
            UserDetailModel detailModel = convertToUserDetailModel(mUser);

            return success(detailModel);
        } catch (Exception ex) {
            ex.printStackTrace();
            return failedSystem();
        }
    }


    @RequestMapping(value = "/users/admin/{user_id}", method = RequestMethod.PUT)
    @ApiOperation(value = "改变用户状态", notes = "根据用户状态改变当前用户状态")
    public boolean activityUser(
            @ApiParam(name = "user_id", value = "id", defaultValue = "")
            @PathVariable(value = "user_id") String userId,
            @ApiParam(name = "activity", value = "激活状态", defaultValue = "")
            @RequestParam(value = "activity") boolean activity) {
        try {
            return userClient.activityUser(userId, activity);
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }


    @RequestMapping(value = "users/password/{user_id}", method = RequestMethod.PUT)
    @ApiOperation(value = "重设密码", notes = "用户忘记密码管理员帮助重新还原密码，初始密码12345678")
    public boolean resetPass(
            @ApiParam(name = "user_id", value = "id", defaultValue = "")
            @PathVariable(value = "user_id") String userId) {
        try {
            return userClient.resetPass(userId);
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }


    @RequestMapping(value = "/users/binding/{user_id}", method = RequestMethod.DELETE)
    @ApiOperation(value = "取消关联绑定", notes = "取消相关信息绑定")
    public boolean unBinding(
            @ApiParam(name = "user_id", value = "", defaultValue = "")
            @PathVariable(value = "user_id") String userId,
            @ApiParam(name = "type", value = "", defaultValue = "")
            @RequestParam(value = "type") String type) {
        try {
            return userClient.unBinding(userId, type);
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }

    /**
     * 重新分配秘钥
     *
     * @param userId 账号
     * @return map  key{publicKey:公钥；validTime：有效时间; startTime：生效时间}
     */
    @RequestMapping(value = "/user/key", method = RequestMethod.PUT)
    @ApiOperation(value = "重新分配密钥", notes = "重新分配密钥")
    public Map<String, String> distributeKey(
            @ApiParam(name = "userId", value = "用户ID", required = true)
            @RequestParam(value = "userId") String userId) {
        try {
            return userClient.distributeKey(userId);
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
    }

    /**
     * 查询用户公钥
     * @param userId
     * @return
     */
    @RequestMapping(value = "/user/key", method = RequestMethod.GET)
    @ApiOperation(value = "查询用户公钥", notes = "查询用户公钥")
    public Envelop getKey(
            @ApiParam(name = "userId", value = "用户ID", required = true)
            @RequestParam(value = "userId") String userId) {
        return userClient.getKey(userId);
    }

    /**
     * 根据登陆用户名及密码验证用户.
     *
     * @param userName
     * @param psw
     */
    @RequestMapping(value = "/users/verification/{login_code}", method = RequestMethod.GET)
    @ApiOperation(value = "根据登陆用户名及密码验证用户", notes = "根据登陆用户名及密码验证用户")
    public Envelop loginVerification(
            @ApiParam(name = "login_code", value = "登录账号", defaultValue = "")
            @PathVariable(value = "login_code") String userName,
            @ApiParam(name = "psw", value = "密码", defaultValue = "")
            @RequestParam(value = "psw") String psw) {
        try {
            MUser mUser = userClient.getUserByNameAndPassword(userName, psw);
            if (mUser == null) {
                return failed("用户信息获取失败!");
            }
            UserDetailModel detailModel = convertToUserDetailModel(mUser);

            return success(detailModel);
        } catch (Exception ex) {
            ex.printStackTrace();
            return failedSystem();
        }
    }

    /**
     * 根据loginCode 获取user
     *
     * @param userName
     * @return
     */
    @RequestMapping(value = "/users/{login_code}", method = RequestMethod.GET)
    @ApiOperation(value = "根据登录账号获取当前用户", notes = "根据登陆用户名及密码验证用户")
    public Envelop getUserByLoginCode(
            @ApiParam(name = "login_code", value = "登录账号", defaultValue = "")
            @PathVariable(value = "login_code") String userName) {
        try {
            MUser mUser = userClient.getUserByUserName(userName);
            if (mUser == null) {
                return failed("用户信息获取失败!");
            }

            UserDetailModel detailModel = convertToUserDetailModel(mUser);

            return success(detailModel);
        } catch (Exception ex) {
            ex.printStackTrace();
            return failedSystem();
        }
    }

    @RequestMapping(value = "/users/existence", method = RequestMethod.GET)
    @ApiOperation(value = "用户属性唯一性验证", notes = "用户属性唯一性验证（用户名、省份证号、邮箱）")
    public Envelop existence(
            @ApiParam(name = "existenceType", value = "", defaultValue = "")
            @RequestParam(value = "existenceType") String existenceType,
            @ApiParam(name = "existenceNm", value = "", defaultValue = "")
            @RequestParam(value = "existenceNm") String existenceNm) {
        try {
            Envelop envelop = new Envelop();
            boolean bo = false;
            switch (existenceType) {
                case "login_code":
                    bo = userClient.isUserNameExists(existenceNm);
                    break;
                case "id_card_no":
                    bo = userClient.isIdCardExists(existenceNm);
                    break;
                case "email":
                    bo = userClient.isEmailExists(existenceNm);
                    break;
                case "telephone":
                    bo = userClient.isTelephoneExists(existenceNm);
                    break;
            }
            envelop.setSuccessFlg(bo);

            return envelop;
        } catch (Exception ex) {
            ex.printStackTrace();
            return failedSystem();
        }

    }


    @RequestMapping(value = "/users/changePassWord", method = RequestMethod.PUT)
    @ApiOperation(value = "修改用户密码", notes = "修改用户密码")
    public Envelop changePassWord(
            @ApiParam(name = "userId", value = "password", defaultValue = "")
            @RequestParam(value = "userId") String userId,
            @ApiParam(name = "password", value = "密码", defaultValue = "")
            @RequestParam(value = "password") String password) {

        Envelop envelop = new Envelop();
        try {
            boolean bo = userClient.changePassWord(userId, password);
            if (bo) {
                envelop.setSuccessFlg(true);
            } else {
                envelop.setSuccessFlg(false);
                envelop.setErrorMsg("密码修改失败");
            }
        } catch (Exception e) {
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg("密码修改失败");
        }
        return envelop;
    }

    /**
     * 将 MUser 转为 UserDetailModel
     *
     * @param mUser
     * @return UserDetailModel
     */
    public UserDetailModel convertToUserDetailModel(MUser mUser) throws Exception{
        if (mUser == null) {
            return null;
        }
        UserDetailModel detailModel = convertToModel(mUser, UserDetailModel.class);

        detailModel.setCreateDate(DateToString(mUser.getCreateDate(), AgAdminConstants.DateTimeFormat));
        detailModel.setLastLoginTime(DateToString(mUser.getLastLoginTime(), AgAdminConstants.DateTimeFormat));

        MConventionalDict dict = null;
        //获取用户类型
        String userType = mUser.getUserType().toString();
        if (StringUtils.isNotEmpty(userType)) {
            int userTypeInt = Integer.parseInt(mUser.getUserType().toString());
            Envelop envStr = roleUserClient.GetUserTypeById(userTypeInt);
            if(envStr != null){
                HashMap userTypeM = objectMapper.readValue(toJson(envStr.getObj()),HashMap.class);
                String userTypeName = userTypeM.get("name").toString();
                detailModel.setUserTypeName(userTypeName);
            }
        }

        //从用户-角色组关系表获取用户所属角色组ids
        detailModel.setRole("");
        Collection<MRoleUser> mRoleUsers = roleUserClient.searchRoleUserNoPaging("userId=" + mUser.getId());
        if (mRoleUsers.size() > 0) {
            StringBuffer buffer = new StringBuffer();
            for (MRoleUser m : mRoleUsers) {
                buffer.append(m.getRoleId());
                buffer.append(",");
            }
            detailModel.setRole(buffer.substring(0, buffer.length() - 1));
        }

        //获取机构信息
        if (StringUtils.isNotEmpty(mUser.getId())) {
            List<MOrgDeptJson> orgDeptJsonList = orgDeptMemberClient.getByUserId(mUser.getId());
            detailModel.setDetailModelList(orgDeptJsonList);
        }

        //获取秘钥信息
        MKey userSecurity = securityClient.getUserKey(mUser.getId(), true);
        if (userSecurity != null) {
            detailModel.setPublicKey(userSecurity.getPublicKey());
            String validTime = DateUtil.toString(userSecurity.getFromDate(), DateUtil.DEFAULT_DATE_YMD_FORMAT)
                    + "~" + DateUtil.toString(userSecurity.getExpiryDate(), DateUtil.DEFAULT_DATE_YMD_FORMAT);
            detailModel.setValidTime(validTime);
            detailModel.setStartTime(DateUtil.toString(userSecurity.getFromDate(), DateUtil.DEFAULT_DATE_YMD_FORMAT));
        }
        return detailModel;
    }

    public MUser convertToMUser(UserDetailModel detailModel) {
        if (detailModel == null) {
            return null;
        }
        MUser mUser = convertToModel(detailModel, MUser.class);
        mUser.setCreateDate(StringToDate(detailModel.getCreateDate(), AgAdminConstants.DateTimeFormat));
        mUser.setLastLoginTime(StringToDate(detailModel.getLastLoginTime(), AgAdminConstants.DateTimeFormat));

        return mUser;
    }

    //查看用户权限列表
    @RequestMapping(value = "/users/user_features", method = RequestMethod.GET)
    @ApiOperation(value = "查看用户权限", notes = "查看用户权限")
    public Envelop getUserFeatureList(
            @ApiParam(name = "roles_ids", value = "用户所属角色组ids，多个以逗号隔开")
            @RequestParam(value = "roles_ids") String roleIds) {
        Collection<MRoleFeatureRelation> relations = roleFeatureRelationClient.searchRoleFeatureNoPaging("roleId=" + roleIds);
        String featureIds = "";
        for (MRoleFeatureRelation relation : relations) {
            featureIds += relation.getFeatureId() + ",";
        }
        if (StringUtils.isEmpty(featureIds)) {
            return success(null);
        }
        featureIds = featureIds.substring(0, featureIds.length() - 1);
        Collection<MAppFeature> mAppFeatures = appFeatureClient.getAppFeatureNoPageSorts("id=" + featureIds, "+sort");
        Envelop envelop = new Envelop();
        List<AppFeatureModel> appFeatureModels = new ArrayList<>();
        for (MAppFeature mAppFeature : mAppFeatures) {
            AppFeatureModel appFeatureModel = convertToModel(mAppFeature, AppFeatureModel.class);
            appFeatureModels.add(appFeatureModel);
        }
        envelop.setSuccessFlg(true);
        envelop.setDetailModelList(appFeatureModels);
        return envelop;
    }

    @RequestMapping(value = "users/getImage/{imageId}", method = RequestMethod.GET)
    @ApiOperation(value = "获取用户信息", notes = "包括地址信息等")
    public String getImage(
            @ApiParam(name = "imageId", value = "", defaultValue = "")
            @PathVariable(value = "imageId") String imageId) {

        try {
            String imagePath = fileResourceClient.getRealPathById(imageId);
            if (imagePath == null) {
                return "用户头像获取失败!";
            }

            return imagePath;
        } catch (Exception ex) {
            ex.printStackTrace();
            return "";
        }
    }

    @RequestMapping(value = ServiceApi.Users.UserPhoneExistence, method = RequestMethod.POST)
    @ApiOperation("获取已存在电话号码")
    public List idExistence(
            @ApiParam(name = "phones", value = "", defaultValue = "")
            @RequestParam("phones") String phones) throws Exception {

        List existPhones = userClient.idExistence(phones);
        return existPhones;
    }

    @RequestMapping(value = ServiceApi.Users.UserOnePhoneExistence, method = RequestMethod.GET)
    @ApiOperation("根据过滤条件判断是否存在")
    public Envelop isExistence(
            @ApiParam(name = "filters", value = "filters", defaultValue = "")
            @RequestParam(value = "filters") String filters) {

        try {
            return success(userClient.isExistence(filters));
        } catch (Exception e) {
            e.printStackTrace();
            return failed("查询出错！");
        }
    }

    @RequestMapping(value = ServiceApi.Users.UserEmailExistence, method = RequestMethod.POST)
    @ApiOperation("获取已存在邮箱")
    public List emailsExistence(
            @ApiParam(name = "emails", value = "", defaultValue = "")
            @RequestParam("emails") String emails) throws Exception {

        List existPhones = userClient.emailsExistence(emails);
        return existPhones;
    }

    /**
     * 根据身份证号获取用户.
     *
     * @param idCardNo
     */
    @RequestMapping(value ="/getPatientInUserByIdCardNo", method = RequestMethod.GET)
    @ApiOperation(value = "根据身份证号获取用户", notes = "根据身份证号获取用户")
    public Envelop getUserByIdCardNo(
            @ApiParam(name = "id_card_no", value = "id_card_no", defaultValue = "")
            @RequestParam(value = "id_card_no") String idCardNo) {
        PatientDetailModel detailModel =new PatientDetailModel();
        try{
            MDemographicInfo demographicInfo = patientClient.getPatient(idCardNo);
            detailModel = patientController.convertToPatientDetailModel(demographicInfo);

        } catch (Exception ex) {
            ex.printStackTrace();
            return failedSystem();
        }
        return success(detailModel);

    }

    @RequestMapping(value = ServiceApi.Users.UseridCardNoExistence,method = RequestMethod.POST)
    @ApiOperation("获取已存在身份证号码")
    public List idCardNoExistence(
            @ApiParam(name = "idCardNos", value = "", defaultValue = "")
            @RequestParam("idCardNos") String idCardNos) throws Exception {

        List existPhones = userClient.idCardNoExistence(idCardNos);
        return existPhones;
    }

    @RequestMapping(value = "systemUsersResetPass/password/{user_id}", method = RequestMethod.PUT)
    @ApiOperation(value = "账户体系-重设密码", notes = "账户体系-密码重置。用户忘记密码管理员帮助重新还原密码，初始密码12345678")
    public Envelop systemUsersResetPass(
            @ApiParam(name = "userId", value = "用户id", defaultValue = "")
            @PathVariable(value = "userId") String userId) {
        Envelop envelop = new Envelop();
        try {
            boolean sussFlag =  userClient.resetPass(userId);
            if(sussFlag){
                envelop.setSuccessFlg(true);
            }
            return envelop;
        } catch (Exception ex) {
            ex.printStackTrace();
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg(ex.getMessage());
            return envelop;
        }
    }


    @RequestMapping(value = "/createSystemUser", method = RequestMethod.POST)
    @ApiOperation(value = "账户体系-创建用户", notes = "账户体系-新增用户")
    public Envelop createSystemUser(
            @ApiParam(name = "userJsonData", value = "用户信息json", defaultValue = "")
            @RequestParam(value = "userJsonData") String userJsonData,
            @ApiParam(name = "registrationType", value = "用户注册方式：默认0为账户注册、1为身份证号注册，2为电话号码注册", defaultValue = "")
            @RequestParam(value = "registrationType") String registrationType) {
        try {
            UserDetailModel detailModel = objectMapper.readValue(userJsonData, UserDetailModel.class);
            String idCard = detailModel.getIdCardNo();
            String telephone = detailModel.getTelephone();
            String errorMsg = null;
            if(StringUtils.isNotEmpty(registrationType)&&registrationType.equals("2")){
                //2为电话号码注册
                if (StringUtils.isEmpty(telephone)) {
                    return failed("电话号码不能为空!");
                }else{
                    detailModel.setLoginCode(telephone);
                }
            }else if(StringUtils.isNotEmpty(registrationType)&&registrationType.equals("1")){
                //1为身份证号注册
                if (StringUtils.isEmpty(idCard)) {
                    return  failed("身份证号不能为空!");
                }else{
                    detailModel.setLoginCode(idCard);
                }
            }else{
                //默认0为账户注册
                if (StringUtils.isEmpty(detailModel.getLoginCode())) {
                    return  failed ("账户不能为空") ;
                }
            }
            if (StringUtils.isEmpty(detailModel.getRealName())) {
                errorMsg += "姓名不能为空!";
            }
            if (StringUtils.isNotEmpty(idCard) && userClient.isIdCardExists(idCard)) {
                return failed("身份证号已存在!");
            }
            if (StringUtils.isNotEmpty(telephone) && userClient.isTelephoneExists(telephone)) {
                return failed("电话号码已存在!");
            }
            if (userClient.isUserNameExists(detailModel.getLoginCode())) {
                return failed("账户已存在!");
            }
            if (StringUtils.isNotEmpty(errorMsg)) {
                return failed(errorMsg);
            }
            MUser mUser = convertToMUser(detailModel);
            //增加居民注册账号时身份证号的校验，demographics表中已存在，users表增加demographic_id身份证号关联
            mUser.setDemographicId(idCard);
            mUser = userClient.createUser(objectMapper.writeValueAsString(mUser));
            if (mUser == null) {
                return failed("保存失败!");
            }
            detailModel = convertToUserDetailModel(mUser);
            return success(detailModel);
        } catch (Exception ex) {
            ex.printStackTrace();
            return failed(ex.getMessage());
        }
    }


    @RequestMapping(value = "/updateSystemUser", method = RequestMethod.PUT)
    @ApiOperation(value = "账户体系-修改用户", notes = "账户体系-修改用户信息")
    public Envelop updateSystemUser(
            @ApiParam(name = "userJsonData", value = "", defaultValue = "")
            @RequestParam(value = "userJsonData") String userJsonData) {
        try {
            UserDetailModel detailModel = toEntity(userJsonData, UserDetailModel.class);
            String errorMsg = "";
            if (StringUtils.isEmpty(detailModel.getLoginCode())) {
                errorMsg += "账户不能为空";
            }
            if (StringUtils.isEmpty(detailModel.getRealName())) {
                errorMsg += "姓名不能为空!";
            }
            if (StringUtils.isEmpty(detailModel.getId())) {
                errorMsg += "id不能为空!";
            }
            if (StringUtils.isNotEmpty(errorMsg)) {
                return failed(errorMsg);
            }
            MUser mUser = userClient.getUser(detailModel.getId());
            if (!mUser.getLoginCode().equals(detailModel.getLoginCode())
                    && userClient.isUserNameExists(detailModel.getLoginCode())) {
                return failed("账户已存在!");
            }
            if (null!= detailModel.getIdCardNo() && !detailModel.getIdCardNo().equals(mUser.getIdCardNo())
                    && userClient.isIdCardExists(detailModel.getIdCardNo())) {
                return failed("身份证号已存在!");
            }
            if (null !=detailModel.getTelephone() && !detailModel.getTelephone().equals(mUser.getTelephone())
                    && userClient.isTelephoneExists(detailModel.getTelephone())) {
                return failed("电话号码已存在!");
            }
            String pass = mUser.getPassword();
            mUser = convertToMUser(detailModel);
            mUser.setPassword(pass);
            mUser.setRole(null);
            mUser = userClient.updateUser(objectMapper.writeValueAsString(mUser));
            if (mUser != null) {
                detailModel = convertToUserDetailModel(mUser);
                return success(detailModel);
            }
            return failed("保存失败！");
        } catch (Exception ex) {
            ex.printStackTrace();
            return failed(ex.getMessage());
        }
    }

    @RequestMapping(value = "/user/picture", method = RequestMethod.POST)
    @ApiOperation(value = "头像上传")
    public Envelop uploadPicture(
            @ApiParam(name = "jsonData", value = "转换后的输入流")
            @RequestParam(value = "jsonData", required = false) String jsonData) throws Exception {
        Envelop envelop = new Envelop();
        //头像上传,接收头像保存的远程路径  path
        String path = null;
        try {
            if (!org.apache.commons.lang.StringUtils.isEmpty(jsonData)) {
                path = userClient.uploadPicture(jsonData);
            }
            envelop.setSuccessFlg(true);
            envelop.setObj("path :"+path);
        }  catch (Exception ex) {
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg(ex.getMessage());
            ex.printStackTrace();
        }
        return  envelop;
    }


    @RequestMapping(value = ServiceApi.Users.InitializeSystemUser, method = RequestMethod.POST)
    @ApiOperation(value = "账户体系-用户注册", notes = "账户体系-用户注册")
    public Envelop initializeSystemUser(
            @ApiParam(name = "userJsonData", value = "用户信息json，默认只有手机号码和身份证号码", defaultValue = "")
            @RequestParam(value = "userJsonData") String userJsonData,
            @ApiParam(name = "registrationType", value = "用户注册方式：默认0为账户注册、1为身份证号注册，2为电话号码注册", defaultValue = "2")
            @RequestParam(value = "registrationType") String registrationType,
            @ApiParam(name = "appId", value = "应用Id,用于关联用户和应用，初始化应用权限", defaultValue = "")
            @RequestParam(value = "appId") String appId,
            @ApiParam(name = "sendMsg", value = "是否发送短信验证码标识（True / false）", defaultValue = "True")
            @RequestParam(value = "sendMsg") String sendMsg) {
        try {
            UserDetailModel detailModel = objectMapper.readValue(userJsonData, UserDetailModel.class);
            String idCard = detailModel.getIdCardNo();
            String telephone = detailModel.getTelephone();
            if(StringUtils.isNotEmpty(registrationType)&&registrationType.equals("2")){
                //2为电话号码注册
                if (StringUtils.isEmpty(telephone)) {
                    return failed("电话号码不能为空!");
                }else{
                    detailModel.setLoginCode(telephone);
                }
            }else{
                return failed("验证方式不正确!");
            }
            detailModel.setRealName(telephone);
            if (StringUtils.isEmpty(detailModel.getRealName())) {
                return failed("姓名不能为空!");
            }
            if (StringUtils.isNotEmpty(telephone) && userClient.isTelephoneExists(telephone)) {
                return failed("电话号码已存在!");
            }
            if (StringUtils.isNotEmpty(idCard) && userClient.isIdCardExists(idCard)) {
                return failed("身份证号已存在!");
            }
            if (userClient.isUserNameExists(detailModel.getLoginCode())) {
                //使用手机号码作为登陆账户
                return failed("电话号码已存在!");
            }
            MUser mUser = convertToMUser(detailModel);
            mUser.setDemographicId(idCard);
            mUser = userClient.createUser(objectMapper.writeValueAsString(mUser));
            //查找应用名称
            MApp mApp = appClient.getApp(appId);
            //保存用户与应用的关联关系
            MUserApp mUserApp = new MUserApp();
            mUserApp.setAppId(appId);
            mUserApp.setUserId(mUser.getId());
            mUserApp.setStatus(0);
            mUserApp.setAppName(mApp.getName());
            mUserApp.setUserName(mUser.getRealName());
            appClient.createUserApp(objectMapper.writeValueAsString(mUserApp));
            if (mUser == null) {
                return failed("保存失败!");
            }
            detailModel = convertToUserDetailModel(mUser);
            return success(detailModel);
        } catch (Exception ex) {
            ex.printStackTrace();
            return failed(ex.getMessage());
        }
    }

    public static boolean isInteger(String str) {
        Pattern pattern = Pattern.compile("^[-\\+]?[\\d]*$");
        return pattern.matcher(str).matches();
    }

}
