认证与授权
====================

> 作者：Sand，2016.02.22

平台安全对象
---------------------

### 用户

所有在平台注册的自然人，根据不同的场景用户类型也会一直增加。

- 患者：平台会先收集患者的基本信息（仅对有身份证号的患者），一方面用于统计平台能够实名跟踪的患者，另一方面用于患者在平台注册并提供身份证号时，平台会快速帮助用户填写这部分基本信息，成为平台用户。患者需要实名认证才可以查阅健康档案，平台提供防身份冒用机制。
- 机构管理员：与特定机构绑定的用户，作为机构管理者。
- 开发者：平台应用开发者。
- 其他：在平台注册的其他角色用户。

**用户凭据**
 
- 用户都有用户名/密码，用于Basic认证
- 用户可以生成私有Token（多个），并限制Token能够访问的数据范围，即Scope。此Token相当于用户在平台上的数据访问钥匙，用户可以公开此Token给第三方使用，并且可以随时作废或修改数据访问权限。
- 平台还会为用户分配一个公钥，可用于数据加解密。用户可根据需要维护此公钥。
 
### 机构

在平台上注册需要与健康之路合作的机构，由工程人员负责收集信息并在平台注册（此信息以后将成为机构门户应用的一部分）。平台会为机构分配一个机构管理员账户，以保证用户属于平台。通过此用户或机构代码可获取机构的信息。

医疗机构是平台健康档案的主要数据来源。

**机构凭据**

- 使用与机构绑定的机构管理员用户名/密码管理机构信息。
- 管理员为机构分配Token（多个），并限制Token能够访问的数据范围，即Scope。
- 平台还会为机构分配一个公钥，可用于数据加解密。平台/机构管理员可根据需要维护此公钥。

### 应用

平台开发者在健康档案平台提交的应用。应用在认证后使用Token调用平台API。

**应用凭据**

- 应用Client ID与Secret
- 应用Token

平台授权体系
---------------------

平台将所有的对象划分为资源及子资源。资源使用Scope标识，即作用域。所有的资源均使用Token访问。根据用户的需求，可以随时修改Token能访问的数据范围。

基于此规则构成平台的整体授权体系。

### Token

Token是平台的授权核心对象。Token与平台资源绑定，设置Token资源访问范围（Scope）可更安全地对外开放数据。Token有以下类型：

- 私有Token（PRIVATE-TOKEN）：平台对象私有的Token，可在“用户个人设置”中心管理Token，私有Token可以有多个，并且可以公开。
- OAuth Token（OAUTH-TOKEN）：由用户授权第三方应用，平台使用OAuth2协议产生的Token。

Token有一个fingerprint，用于区别同一个用户使用同一个应用创建的不同Token。

### Scope

作用域，限制Token可访问数据范围/资源范围。用户可以根据Token随时修改其作用域，达到权限控制的目的。

授权模式
---------------------

平台支持两种授权模式，两种授权模式各用于不同的业务场景：

> [Basic授权(用户名/密码)](basic-auth.html)

> [OAuth2授权](oauth2-auth.html)

对OAuth2授权，需要限定开发者，用户（狭义，指非开发者用户）属于平台，而不是属于第三方应用，否则用户无法在平台上对应用进行授权。

总结平台API调用授权情况：

- 不需授权，如API根路径：https://ehr.yihu.com/api
- 需要授权的API
	- Basic授权
	- OAuth授权

健康之路应用
---------------------

目前平台对接了集成开放平台，转诊及小薇社区三个应用。

### 集成开放平台

集成开放平台用于医疗机构数据上传，所有医疗机构部署的是同一个应用（Client ID一致），即不同的医院，用的是同一个数据采集客户端。
但集成开放平台的用户不属于健康档案平台的开发者，集成开放平台不使用OAuth Web授权流程，因此平台无法对用户使用OAuth授权。
并且数据上传行为并不是访问平台的特定资源，因此集成开放平台调用健康档案平台API时不走OAuth2认证，而是使用Basic认证。

**流程**

- 现场人员使用健康档案平台为“机构”分配的用户A与密码S登录集成开放平台（即机构的管理员）。
- 集成开放平台使用A/S组合，通过平台API验证是否正确。如果正确，平台返回一个临时Token，用于应用与平台之间的交互（Token过期时间为20分钟，可刷新）。
- 集成开放平台使用用户名与Token获取机构信息，初始化应用的机构信息。
- 集成开放平台使用Token调用其他业务API，平台负责对所有的请求校验此Token，若Token无效或过期，会返回错误信息。

### 转诊应用

转诊应用向平台传递转诊数据包。转诊应用是唯一的，但转诊应用的用户同样是平台用户，且应用只是向平台输送数据，并且转诊应用的用户并非平台用户。
同样无法使用OAuth Web授权流程，而使用Basic认证。

**数据包上传流程**

- 转诊应用使用健康档案平台为“健康之路”分配的管理员用户名/密码登录。
- 平台验证用户名/密码组合，如果正确返回一个临时Token（Token过期时间为20分钟，可刷新）。
- 转诊应用使用Token上传转诊数据/调阅用户档案，平台负责对所有的请求校验此Token，若Token无效或过期，会返回错误信息。
